// CQGCELEventHandlers.cpp : implementation file
//

#include "stdafx.h"
#include <stdexcept>
#include <cassert>
#include "Common.h"
#include "QFormulasDlg.h"
#include "StrategyOrderPlacing.h"
#include "StrategyOrderPlacingDlg.h"

namespace
{
   /// \brief Shows message box with symbol resolution failed reason.
   /// \param symbol - Requested symbol
   /// \param cqg_error - Error occured during a symbol resolution process
   void onSymbolResolutonFailed(BSTR symbol, ICQGError* cqg_error)
   {
      ATL::CComBSTR errDescription;
      auto hr = cqg_error->get_Description(&errDescription);
      AssertCOMError<ICQGError>(cqg_error, hr);

      CStringW msg;
      msg.AppendFormat(L"%s resolution failed. %s", symbol, errDescription.m_str);
      AfxMessageBox(ATL::CW2T(msg), MB_ICONWARNING);
   }
}

STDMETHODIMP CStrategyOrderPlacingDlg::OnDataError(
                                 LPDISPATCH /*obj*/,
                                 BSTR errorDescription)
{
   CString errMsg("Data error: ");
   errMsg += ATL::CW2T(errorDescription);
   AfxMessageBox(errMsg);

   return S_OK;
}

STDMETHODIMP CStrategyOrderPlacingDlg::OnGWConnectionStatusChanged(
                                 eConnectionStatus newStatus)
{
   try
   {
      m_lblGWConnection.SetWindowText(_T("GW connection ") + ConnectionStatusToString(newStatus));

      if (newStatus == csConnectionUp)
      {
         m_spCQGCEL->put_AccountSubscriptionLevel(aslAccountUpdatesAndOrders);
         m_GWConnectionUp = true;
      }
      else
      {
         m_GWConnectionUp = false;
      }
      
      EnableControls();
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CStrategyOrderPlacingDlg::OnDataConnectionStatusChanged(
                                 eConnectionStatus newStatus)
{
   try
   {
      CString status;
      
      switch (newStatus)
      {
         case csConnectionDelayed:
            status = _T("delayed");
            break;
         case csConnectionDown:
            status = _T("down");
            break;
         case csConnectionUp:
            status = _T("up");
            break;
         default:
            assert(!"Unexpected data connection status");
      }
      m_lblDataConnection.SetWindowText(_T("Data connection ") + status);

      m_DataConnectionUp = (newStatus == csConnectionUp);
   
      EnableControls();
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CStrategyOrderPlacingDlg::OnAccountChanged(
                                 eAccountChangeType change, 
                                 ICQGAccount* /*account*/, 
                                 ICQGPosition* /*position*/)
{
   try
   {
      if (change == actAccountsReloaded)
      {
         ReloadAccounts();
      }
   }
   catch (std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CStrategyOrderPlacingDlg::OnInstrumentResolved(
   BSTR symbol, ICQGInstrument* instrument, ICQGError* cqg_error)
{
   try
   {
      VARIANT_BOOL errorOccured = VARIANT_FALSE;
      auto hr = m_spCQGCEL->IsValid(ATL::CComVariant(cqg_error), &errorOccured);
      AssertCOMError<ICQGError>(cqg_error, hr);

      if (errorOccured)
      {
         onSymbolResolutonFailed(symbol, cqg_error);
         return S_OK;
      }

      ATL::CComBSTR fullName;
      hr = instrument->get_FullName(&fullName);
      AssertCOMError<ICQGInstrument>(instrument, hr);

      // If instrument is already subscribed do not add it to the list
      int row = m_cmbSpreadExpr.FindStringExact(0, ATL::CW2T(fullName));
      if(row == CB_ERR)
      {
         m_cmbSpreadExpr.InsertString(0, ATL::CW2T(fullName));
         m_cmbSpreadExpr.SelectString(0, ATL::CW2T(fullName));
      }
      m_cmbSpreadExpr.SelectString(row, ATL::CW2T(fullName));
      UpdateDefinitionInfo();
      EnableControls();
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CStrategyOrderPlacingDlg::OnInstrumentChanged(
                                 ICQGInstrument* instrument, 
                                 ICQGQuotes* /*quotes*/, 
                                 ICQGInstrumentProperties* /*props*/)
{
   try
   {
      // Get selected instrument
      ATL::CComPtr<ICQGInstrument> spSelInstrument = GetSelectedInstrument();

      if (spSelInstrument == NULL)
      {
         // No instrument is selected
         return S_OK;
      }

      // Get changed instrument's full name
      ATL::CComBSTR fullName;
      HRESULT hr = instrument->get_FullName(&fullName);
      AssertCOMError<ICQGInstrument>(instrument, hr);

      // Get selected instrument's full name
      ATL::CComBSTR selFullName;
      GET_PARAM(spSelInstrument, selFullName, get_FullName);
   
      if (fullName == selFullName)
      {
         // Selected instrument was changed, show it's quotes
         ShowQuotes(instrument);
      }
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CStrategyOrderPlacingDlg::OnOrderChanged(
                                 eChangeType change, 
                                 ICQGOrder* order, 
                                 ICQGOrderProperties* /*oldProperties*/, 
                                 ICQGFill* /*fill*/, 
                                 ICQGError* /*cqgerr*/)
{
   try
   {
      // Get order's account
      ATL::CComPtr<ICQGAccount> spAccount;
      HRESULT hr = order->get_Account(&spAccount);
      AssertCOMError<ICQGOrder>(order, hr);

      // Get order's account ID 
      long orderAccID;
      GET_PARAM(spAccount, orderAccID, get_GWAccountID);

      // Get selected account ID
      long selAccID = static_cast<long>(m_cmbAccounts.GetItemData(m_cmbAccounts.GetCurSel()));

      if (orderAccID != selAccID)
      {
         // Order's account is not selected
         return S_OK;
      }

      // Add or update order in the list control
      ShowStrategyOrder(order, change);
      ShowLegOrder(order, change);

      // Update and show position
      UpdatePosition(order);  
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}


STDMETHODIMP CStrategyOrderPlacingDlg::OnEventStrategyDefinitionProgress(
                                    ICQGStrategyDefinition* definition,
                                    ICQGError* /*cqgerr*/)
{
   try
   {
      UpdateDefinitionHolder(definition);
      ClearAllInstrumentData();
      ShowValidPrice(GetSelectedInstrument());
      ShowStrategyProperties(definition);
      EnableControls();
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CStrategyOrderPlacingDlg::OnQueryProgress(
                                    ICQGOrdersQuery* query, 
                                    ICQGError* /*error*/)
{
   try
   {
      UpdatePositions(query);
      ShowPosition();
   }
   catch (std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CQFormulasDlg::OnQFormulaDefinitionsResolved(
   ICQGQFormulaDefinitions* cqg_qformula_definitions,
   ICQGError* cqg_error)
{
   try
   {
      // Raise error, when any error occurred during request
      VARIANT_BOOL isValid;
      HRESULT hr = m_spCQGCEL->IsValid(ATL::CComVariant(cqg_error), &isValid);
      AssertCOMError(m_spCQGCEL, hr);

      if (isValid == VARIANT_TRUE)
      {
         ATL::CComBSTR errDescription;
         hr = cqg_error->get_Description(&errDescription);
         AssertCOMError<ICQGError>(cqg_error, hr);

         AfxMessageBox(ATL::CW2T(errDescription), MB_ICONEXCLAMATION);

         return S_OK;
      }

      m_spQFormulaDefinitions = cqg_qformula_definitions;

      // Fill up new data
      ClearAllData();

      long count;
      hr = cqg_qformula_definitions->get_Count(&count);
      AssertCOMError<ICQGQFormulaDefinitions>(cqg_qformula_definitions, hr);

      for (long i = 0; i < count; ++i)
      {
         ATL::CComPtr<ICQGQFormulaDefinition> spQFormulaDefinition;
         hr = cqg_qformula_definitions->get_Item(i, &spQFormulaDefinition);
         AssertCOMError<ICQGQFormulaDefinitions>(cqg_qformula_definitions, hr);

         ATL::CComBSTR name;
         hr = spQFormulaDefinition->get_Name(&name);
         AssertCOMError(spQFormulaDefinition, hr);

         const int row = m_cmbDefinition.AddString(ATL::CW2T(name));
         m_cmbDefinition.SetItemData(row, i);
      }

      if (count > 0)
      {
         // Select first definition
         m_cmbDefinition.SetCurSel(0);
         OnCbnSelchangeCmbDefinition();
      }
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}